# Validation

We describe validation here only for the frontend.

The *real* validation needs to be done in the backend.
A user can easily change the html code e.g. to enable a button or event send forged/changed requests to the backend.

Because we currently only apply validation to dialogs we will explain how validation works with them

**Note that there is currently a pattern for after type conversion validation only.**

e.g. if the types needs a number but the user inputs just text this cannot be validated currently.

Also note that currently you can only return one validation message even though you can specify multiple validation functions

## Dialogs

You will need two more objects for validation

- one for the validation rules
- one for the validation messages

place these inside the reducer of the dialog state.

```ts

import isPositiveNumber from ... validation functions

export type State = {
  readonly id: number
  readonly name: string
  readonly age: number
}

export const validationRules = getValidationCollection<State>({
  id: [],
  name: [(_name) => name !== null || name !== "" ], //_name is implicitly typed string because name in state also have string type
  age: [(_age) => age !== null, isPositiveNumber],
})

export const validationMessageKeys = getValidationMessagesCollection<State>({
  id: '',
  name: 'Name must not be empty',
  age: 'Age must be larger than 0',
})
```

`getValidationCollection` is a helper function to ensure that every property of the state has a corresponding property in the `validationRules` and that the validation function gets the right type as input

`getValidationMessagesCollection` does basically the same thing but ensures that the property values of `validationMessageKeys` are strings (the messages in case we got an invalid value)


Luckily we implemented the validation into the `MaterialInput` components so you just have to use

```ts
import {validationRules, validationMessageKeys} from ... the reducer ...

//in the component

render(): JSX.Element {
  return (
  <div>
    <MaterialInput
      value={this.props.name}
      validations={validationRules.name}
      validationMessageKey={validationMessageKeys.name}
      onChange={(e) => this.props.setName(e.currentTarget.value)}
    />

    <MaterialInput
      type="number"
      value={this.props.age}
      validations={validationRules.age}
      validationMessageKey={validationMessageKeys.age}
      onChange={(e) => {
        const time = parseInt(e.currentTarget.value, 10)

        this.props.setAge(isNaN(time)
        ? null
        : time)

      })}
    />
  </div>
})
```

And thats it
